<?php
namespace Qihoo\Lib;

use Qihoo\Lib\Response;
use Qihoo\Lib\Logger\LogSvc;

class Request
{
    static public $connect_timeout = 2;
    static public $execute_timeout = 2;

    public static function get($host, $url, $auth, $headers = array(), $is_download = false)
    {/*{{{*/
        return self::request($host, $url, $auth, $headers, null, $is_download);
    }/*}}}*/

    public static function post($host, $url, $auth, $post, $headers = array())
    {/*{{{*/
        return self::request($host, $url, $auth, $headers, $post);
    }/*}}}*/

    private static function request($host, $url, $auth, $headers, $post, $is_download = false, $fname = null)
    {/*{{{*/
        if(\substr($host, 0, 7) == 'http://'){
            $domain = \substr($host, 7);
        } else {
            $domain = $host;
        }
        
        if($auth != null){
            $auth_msg = $auth->authorization($url);
            $headers['Authorization'] = $auth_msg;
        }

        $headerArr = array(); 
        foreach($headers as $n=>$v){ 
            $headerArr[] = $n .':' . $v;  
        }
        $ch = \curl_init($host.$url);

        \curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // 获取数据返回
        \curl_setopt($ch, CURLOPT_BINARYTRANSFER, true); // 在启用 CURLOPT_RETURNTRANSFER 时候将获取数据返回
        \curl_setopt($ch, CURLOPT_HTTPHEADER, $headerArr);
        \curl_setopt($ch, CURLOPT_HEADER, 1);
        \curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        \curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, self::$connect_timeout);
        \curl_setopt($ch, CURLOPT_TIMEOUT, self::$execute_timeout);

        if(isset($post)){
            \curl_setopt($ch, CURLOPT_POST, true);
            \curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
        }

        $beginTime = \array_sum(explode(' ', \microtime()));

        $output = \curl_exec($ch);
        $errno  = \curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if($errno == 0) {
            $curl_no  = \curl_errno($ch);
            $curl_msg = \curl_error($ch);
        }   
        $header_size = \curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        \curl_close($ch);

        $endTime = \array_sum(explode(' ', \microtime()));

        if($is_download){
            \header('Content-Type: application/octet-stream');
            \header('Content-Transfer-Encoding: binary');
            \header('Content-Disposition: attachment; filename="'. $fname. '"');
        }

        $output_headers = self::parseHeaders($output, $header_size);
        $output         = \substr($output, $header_size);

        $response = new Response($errno, $endTime - $beginTime, $output_headers, $output);
        if($errno == 0) {
            $response->headers['errno']  = $curl_no;
            $response->headers['errmsg'] = $curl_msg;
        }

        \constant('\Qihoo\Config::QIHOO_LOG')&&self::log($errno, \strlen($output), 
            $response->duration, $host, $url, $response->headers['errno'], $response->headers['errmsg'],$headers, $post);
        return $response;
    }/*}}}*/

    public static function uploadData($host, $url, $file_name, $file_body, $auth, $fields, $mimeType = null, array $headers = array())
    {/*{{{*/
        $data = array();
        $mimeBoundary = '----'.\md5(microtime());

        foreach ($fields as $key => $val) {
            \array_push($data, '--' . $mimeBoundary);
            \array_push($data, "Content-Disposition: form-data; name=\"$key\"");
            \array_push($data, '');
            \array_push($data, $val);
        }

        \array_push($data, '--' . $mimeBoundary);
        $mimeType = empty($mimeType) ? 'application/octet-stream' : $mimeType;
        $file_name = self::escapeQuotes($file_name);
        \array_push($data, "Content-Disposition: form-data; name=\"file\"; filename=\"$file_name\"");
        \array_push($data, "Content-Type: $mimeType");
        \array_push($data, '');
        \array_push($data, $file_body);

        \array_push($data, '--' . $mimeBoundary . '--');

        $body = \implode("\r\n", $data);
        $contentType = 'multipart/form-data; boundary=' . $mimeBoundary;
        $headers['Content-Type'] = $contentType;
        $headers['Content-Length'] = \strlen($body);
        return self::post($host, $url, $auth, $body, $headers);
    }/*}}}*/

    private static function escapeQuotes($str)
    {/*{{{*/
        $find = array("\\", "\"");
        $replace = array("\\\\", "\\\"");
        return \str_replace($find, $replace, $str);
    }/*}}}*/

    private static function log($nginx_code = 0, $len = 0, $duration = 0, $host = '', $url = '', $errno = 0, $errmsg = '', $header = null, $post = null)
    {/*{{{*/
        $info = array();
        $info['time']       = \date('[d/M/Y:H:i:s O]', \time());
        if($post == null){
            $info['type']   = '[GET]';
        } else {
            $info['type']   = '[POST]';
        }
        $info['nginx_code'] = $nginx_code; 
        $info['len']        = $len; 
        $info['consume']    = (int)\round($duration * 1000, 1);
        $info['host']       = $host;
        $info['url']        = $url;
        $info['errno']      = $errno;
        $info['errmsg']     = $errmsg;
        $info['header']     = $header;
        $info['post']       = $post;

        $message = \json_encode($info) . "\n";

        LogSvc::getOssRequestLog()->log($message); 
    }/*}}}*/

    private static function parseHeaders($output, $header_size)
    {/*{{{*/
        $raw = \substr($output, 0, $header_size);
        $headers = array();
        $headerLines = \explode("\r\n", $raw);
        foreach ($headerLines as $line) {
            $headerLine = \trim($line);
            $kv = \explode(':', $headerLine);
            if (\count($kv) >1) {
                $key = $kv[0];
                unset($kv[0]);
                $headers[$key] = \trim(implode(':', $kv));
            }
        }
        return $headers;
    }/*}}}*/

}
