<?php
namespace Qihoo\Lib;

class Response
{
    public $statusCode;
    public $headers;
    public $body;
    public $error;
    private $jsonData;
    public $duration;

    /** @var array Mapping of status codes to reason phrases */
    private static $statusTexts = array
        (/*{{{*/
        100 => 'Continue',
        101 => 'Switching Protocols',
        102 => 'Processing',
        200 => 'OK',
        201 => 'Created',
        202 => 'Accepted',
        203 => 'Non-Authoritative Information',
        204 => 'No Content',
        205 => 'Reset Content',
        206 => 'Partial Content',
        207 => 'Multi-Status',
        208 => 'Already Reported',
        226 => 'IM Used',
        300 => 'Multiple Choices',
        301 => 'Moved Permanently',
        302 => 'Found',
        303 => 'See Other',
        304 => 'Not Modified',
        305 => 'Use Proxy',
        307 => 'Temporary Redirect',
        308 => 'Permanent Redirect',
        400 => 'Bad Request',
        401 => 'Unauthorized',
        402 => 'Payment Required',
        403 => 'Forbidden',
        404 => 'Not Found',
        405 => 'Method Not Allowed',
        406 => 'Not Acceptable',
        407 => 'Proxy Authentication Required',
        408 => 'Request Timeout',
        409 => 'Conflict',
        410 => 'Gone',
        411 => 'Length Required',
        412 => 'Precondition Failed',
        413 => 'Request Entity Too Large',
        414 => 'Request-URI Too Long',
        415 => 'Unsupported Media Type',
        416 => 'Requested Range Not Satisfiable',
        417 => 'Expectation Failed',
        422 => 'Unprocessable Entity',
        423 => 'Locked',
        424 => 'Failed Dependency',
        425 => 'Reserved for WebDAV advanced collections expired proposal',
        426 => 'Upgrade required',
        428 => 'Precondition Required',
        429 => 'Too Many Requests',
        431 => 'Request Header Fields Too Large',
        500 => 'Internal Server Error',
        501 => 'Not Implemented',
        502 => 'Bad Gateway',
        503 => 'Service Unavailable',
        504 => 'Gateway Timeout',
        505 => 'HTTP Version Not Supported',
        506 => 'Variant Also Negotiates (Experimental)',
        507 => 'Insufficient Storage',
        508 => 'Loop Detected',
        510 => 'Not Extended',
        511 => 'Network Authentication Required',
    );/*}}}*/

    const BODY_TYPE_RAW  = 0;
    const BODY_TYPE_JOSN = 1;

    public function __construct($code, $duration, array $headers = array(), $body = null, $error = null)
    {/*{{{*/
        $this->statusCode = $code;
        $this->duration   = $duration;
        $this->headers    = $headers;
        $this->body_type  = self::BODY_TYPE_RAW;

        if(!isset($this->headers['errno'])){
            $this->headers['errno'] = -1;
        }
        if(!isset($this->headers['errmsg'])){
            $this->headers['errmsg'] = '';
        }

        if($this->_is_json($body)){
            $this->body_type = self::BODY_TYPE_JOSN;
            $this->body = json_decode($body, true);
        } else {
            $this->body_type = self::BODY_TYPE_RAW;
            $this->body = $body;
        }
        $this->error    = $error;
        $this->jsonData = $body;

        if($error != null){
            return;
        }

        if($this->statusCode >= 200 && $this->statusCode < 300){
            $this->error = null;
            return;
        }
        if($body == null){
            if ($code >= 400) {
                $this->error = self::$statusTexts[$code];
            }
            return;
        }
        if(isset($this->jsonData['error']) ){
            $this->error = $this->jsonData['error'];
        } else {
            $this->error = null;
        }
    }/*}}}*/

    public function json()
    {/*{{{*/
        return $this->jsonData;
    }/*}}}*/

    private function _is_json($string) 
    {/*{{{*/
        json_decode($string);
        return (json_last_error() == JSON_ERROR_NONE);
    }/*}}}*/

    private static function isJson($headers)
    {/*{{{*/
        return isset($headers['Content-Type']) &&
        strpos($headers['Content-Type'], 'application/json') === 0;
    }/*}}}*/
}
