<?php
namespace Qihoo\Oss;
use Qihoo\Lib\Request;
use Qihoo\AreaHost;
use Qihoo\Lib\Util;

class ObjectManager
{

    public function upload($auth, $bucket, $filePath, $fileName, $fileType)
    {/*{{{*/
        $deadline = \time() + 3600;
        $params = array(
            'bucket' => $bucket,
            'object' => $fileName,
            'deadline' => $deadline,
        );
        $str   = Util::urlBase64Encode(\json_encode($params));
        $token = $auth->accessKey.':'.md5($str.$auth->secretKey).':'.$str;

        if(\version_compare(PHP_VERSION, '5.5.0') >= 0){
            $file = new \CURLFile($filePath, $fileType, $fileName); 
        } else {
            $file = '@'.$filePath;
        }
        $host = AreaHost::getIns()->getUpHost();
        $url  = '/Object.upload';
        $post = array('file'=>$file, 'token' => $token);

        return Request::post($host, $url, null, $post);
    }/*}}}*/

    public function getDownloadUrl($auth, $bucket, $fileName, $expire)
    {/*{{{*/
        $fileName = Util::urlBase64Encode($fileName);
        $expire   = $expire + time();
        $url = '/Object.getDownloadUrl/'.$bucket.'/'.$fileName.'?e='.$expire;
        return Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
    }/*}}}*/
    
    public function getDownloadUrls($auth, $bucket, $files, $expire)
    {/*{{{*/
        $expire = $expire + time();
        $url   = '/Object.getDownloadUrls/'.$bucket.'?e='.$expire;
        $files = Util::urlBase64Encode(json_encode($files));
        $postBody = 'files='.$files;
        $token  = $auth->accessKey.':'.md5($url.'\n'.$postBody.$auth->secretKey);
        $header = array('Authorization' => $token);
        return Request::post(AreaHost::getIns()->getApiHost(), $url, null, $postBody, $header);
    }/*}}}*/

    public function stat($auth, $bucket, $fileName)
    {/*{{{*/
        $fileName = Util::urlBase64Encode($fileName);
        $url = '/Object.stat/'.$bucket.'/'.$fileName;
        return Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
    }/*}}}*/
    
    public function delete($auth, $bucket, $fileName)
    {/*{{{*/
        $fileName = Util::urlBase64Encode($fileName);
        $url = '/Object.delete/'.$bucket.'/'.$fileName;
        return Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
    }/*}}}*/
    
    public function move($auth, $srcBucket, $srcFile, $dstBucket, $dstFile)
    {/*{{{*/
        $srcFile = Util::urlBase64Encode($srcFile);
        $dstFile = Util::urlBase64Encode($dstFile);
        $url = '/Object.move/'.$srcBucket.'/'.$srcFile .'/'.$dstBucket .'/' . $dstFile;
        return Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
    }/*}}}*/
    
    public function copy($auth, $srcBucket, $srcFile, $dstBucket, $dstFile)
    {/*{{{*/
        $srcFile = Util::urlBase64Encode($srcFile);
        $dstFile = Util::urlBase64Encode($dstFile);
        $url = '/Object.copy/'.$srcBucket.'/'.$srcFile .'/'.$dstBucket .'/' . $dstFile;
        return Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
    }/*}}}*/

    public function getList($auth, $bucket, $prefix, $page, $limit)
    {/*{{{*/
        $url = '/Object.getList/'.$bucket.'/?page='.$page.'&limit=' . $limit .'&prefix=' . $prefix;
        return Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
    }/*}}}*/

    public function multiUpload($auth, $bucket, $filePath, $fileName, $parallel)
    {/*{{{*/
        $fileSize = \filesize($filePath);
        $encodeFileName = Util::urlBase64Encode($fileName);
        $requestUrl     =  '/request/'.$bucket.'/'.$encodeFileName.'?fsize='.$fileSize.'&parallel='.$parallel;
        $requestRet     = Request::get(AreaHost::getIns()->getUpHost(), $requestUrl, $auth);
        $requestInfo    = $requestRet->body;
        $blockHeads     = $requestInfo['block_heads'];
        $commitUrl      = $requestInfo['commit_url'];
        //这里支持多进程分片上传
        $fileHandle     = \fopen($filePath, 'r');
        foreach ($blockHeads as $blockInfo) {
            $this->_blockUpload($blockInfo, $fileHandle);
        }
        fclose($fileHandle);
        return $this->_commitUpload($commitUrl);
    }/*}}}*/

    public function multiUploadV2($auth, $bucket, $filePath, $fileName, $parallel)
    {/*{{{*/
        $fileSize = \filesize($filePath);
        $params   = array(
            'bucket' => $bucket,
            'object' => $fileName,
            'fsize'  => $fileSize,
            'parallel' => $parallel,
        );
        $str   = Util::urlBase64Encode(\json_encode($params));
        $token = $auth->accessKey.':'.md5($str.$auth->secretKey).':'.$str;
        $url   = '/Object.request/';
        $header = array('Authorization' => $token);
        $requestInfo    = Request::post(AreaHost::getIns()->getUpHost(), $url, null, null, $header);
        $requestInfo    = $requestInfo->body;
        $blockHeads     = $requestInfo['block_heads'];
        $commitUrl      = $requestInfo['commit_url'];
        //这里支持多进程分片上传
        $fileHandle     = \fopen($filePath, 'r');
        foreach ($blockHeads as $blockInfo) {
            $this->_blockUpload($blockInfo, $fileHandle);
        }
        return $this->_commitUpload($commitUrl);
    }/*}}}*/
    
    public function getFile($auth, $bucket, $fileName)
    {/*{{{*/
        $fileName = Util::urlBase64Encode($fileName);
        $url = '/Object.getDownloadUrl/'.$bucket.'/'. $fileName;
        $fileInfo = Request::get(AreaHost::getIns()->getApiHost(), $url, $auth);
        $fileInfo = $fileInfo->body;
        $downloadUrl = $fileInfo['downloadUrl'];
        return Request::get('', $downloadUrl, null);
    }/*}}}*/
    
    //SDK不对外开放此API，API更适合支持302跳转的客户端
//    public function getPreview($auth, $bucket, $fileName, $expire, $width, $height)
//    {/*{{{*/
//        $fileName = Util::urlBase64Encode($fileName);
//        $url = '/Object.getPreview/'.$bucket.'/'. $fileName . '?e=' . $expire .
//            '&width='. $width .'&height='. $height;
//        $auth_msg = $auth->authorization($url);
//        $url .= '&Authorization='.$auth_msg;
//        return Request::get(AreaHost::getIns()->getApiHost(), $url, null);
//    }/*}}}*/
    
    private function _blockUpload($blockInfo, $fileHandle)
    {/*{{{*/
        if (!isset($blockInfo['is_last_block']) || $blockInfo['is_last_block']) {
            return;
        }
        $begin = $blockInfo['begin'];
        $end   = $blockInfo['end'];
        $url   = $blockInfo['next_block_url'];

        if ($fileHandle && 0 === \fseek($fileHandle, $begin)) {
            $blockData = \fread($fileHandle, $end - $begin + 1);    // 注意加1
            $blockHash = \sha1($blockData);
            $nextBlock = Request::uploadData('', $url, 'block', $blockData, null, array('bhash' => $blockHash));
            $this->_blockUpload($nextBlock->body, $fileHandle);
        } else {
            return;
        }
    }/*}}}*/

    private function _commitUpload($commitUrl)
    {/*{{{*/
        return Request::get('', $commitUrl, null);
    }/*}}}*/

}
