<?php
namespace Qihoo;

use Qihoo\Lib\Auth;
use Qihoo\Config;
use Qihoo\Lib\Logger\LogSvc;
use Qihoo\Lib\Response;

final class OssSvc
{
    private $auth;
    private $manager = array('Object');
    const HTTP_STATUS_OK = 200;
    const HTTP_STATUS_CONTINUE = 100;
    const DEF_DOWNLOAD_URL_EXPIRE = 43200;      //60 * 60 * 12

    public function __construct($akey, $skey, $area = '') 
    {/*{{{*/
        $auth = new Auth($akey, $skey);
        $this->auth = $auth;

        if(!empty($area)) {
            Config::$QIHOO_OSS_AREA = $area;
        }
        $logPath = Config::QIHOO_LOG_PATH;
        if(empty($logPath)) {
            $logPath = \dirname(__FILE__).'/logs/';
        }
        LogSvc::setLogPath($logPath);
    }/*}}}*/

    public function __call($method, $arguments)
    {/*{{{*/
        $objProxy = $this->getClassObj($method);
        if($objProxy === false) {
            $message = '[Method Wrong] method:'. $method . ' args:' . \json_encode($arguments); 
            \constant('\Qihoo\Config::QIHOO_LOG')&&LogSvc::getOssErrLog()->log($message);
            return false;
        }

        \array_unshift($arguments, $this->auth);

        return $this->parseResponse(\call_user_func_array($objProxy, $arguments), $objProxy);
    }/*}}}*/

    private function getClassObj($method)
    {/*{{{*/
        foreach($this->manager as $manager) {
            $head = \substr($method, 0, \strlen($manager));
            if(\strlen($head) < \strlen($manager)) {
                continue;
            }
            
            if($head == \strtolower($manager)) {
                $methodName = \lcfirst(\substr($method, \strlen($manager)));

                $manager = '\Qihoo\Oss\\' .$manager. 'Manager';
                $obj = new $manager();
                return array($obj, $methodName);
            }
        }
        return false;
    }/*}}}*/

    //处理结果为兼容Object的返回结果
    private function parseResponse($res, $objProxy) 
    {/*{{{*/
        $className = get_class($objProxy[0]);
        if($className != 'Qihoo\Oss\ObjectManager'&&$res->statusCode != self::HTTP_STATUS_OK && $res->statusCode !=self::HTTP_STATUS_CONTINUE) {
            return \json_encode(['errorCode'=>$res->statusCode, 'errorMessage'=>$res->headers['errmsg']]);
        }
        if ($res->body_type === Response::BODY_TYPE_JOSN) {
            return \json_encode($res->body);
        } else {
            return $res->body;
        }
    }/*}}}*

    /*
     *上传小文件，最好不要超过20M，太大建议使用分片
     */
    public function uploadObject($bucket, $filePath, $fileName, $fileType=NULL)
    {/*{{{*/
        return $this->objectUpload($bucket, $filePath, $fileName, $fileType);
    }/*}}}*/

    /*
     *获取对象的下载地址
     */
    public function getDownloadUrlForObject($bucket, $fileName, $expire=self::DEF_DOWNLOAD_URL_EXPIRE)
    {/*{{{*/
        return $this->objectGetDownloadUrl($bucket, $fileName, $expire); 
    }/*}}}*/
        
    /*
     *批量获取对象的下载地址
     */
    public function getDownloadUrlsForObject($bucket, $files, $expire=self::DEF_DOWNLOAD_URL_EXPIRE)
    {/*{{{*/
        return $this->objectGetDownloadUrls($bucket, $files, $expire);
    }/*}}}*/


    /*
     *获取对象的状态
     */
    public function statObject($bucket, $fileName)
    {/*{{{*/
        return $this->objectStat($bucket, $fileName); 
    }/*}}}*/
        
    /*
     *删除对象
     */
    public function deleteObject($bucket, $fileName)
    {/*{{{*/
        return $this->objectDelete($bucket, $fileName); 
    }/*}}}*/
        
    /*
     *复制对象
     */
    public function copyObject($srcBucket, $srcFile, $dstBucket, $dstFile)
    {/*{{{*/
        return $this->objectCopy($srcBucket, $srcFile, $dstBucket, $dstFile);
    }/*}}}*/
        
    /*
     *移动对象
     */
    public function moveObject($srcBucket, $srcFile, $dstBucket, $dstFile)
    {/*{{{*/
        return $this->objectMove($srcBucket, $srcFile, $dstBucket, $dstFile);
    }/*}}}*/
        
    /*
     *前缀的方式获取批量对象
     */
    public function getListForObject($bucket, $prefix, $page = 1, $limit = 10)
    {/*{{{*/
        return $this->objectGetList($bucket, $prefix, $page, $limit);
    }/*}}}*/
        
    /*
     *获取对象数据,不建议对外提供
     */
    public function getFileForObject($bucket, $fileName, $expire = self::DEF_DOWNLOAD_URL_EXPIRE)
    {/*{{{*/
        return $this->objectGetFile($bucket, $fileName, $expire);
    }/*}}}*/
    
}
